local addonName, L = ...;

local FLAMEBUFFICD = 0.578; -- approx. milliseconds (determined by experiment rather than calculation)

-- default settings
local defaults = {
					soundfile = "Sound\\Doodad\\SimonGame_LargeGreenTree.wav",
					buffname = "Burning Infusion",
					heatwarn = 90,
                    debugmode = false,
                 };
local lastgain,lastcount = 0,0;
local overheat = false;
local idle = true;
local enabled = true;

-- not local, called back by the timer function
function ICDExpired()
	if (FAVars.debugmode) then print("FA: icd over"); end
	if (not overheat) then
		FAFrameReady:Show();
		FAFrameBeware:Hide();
	else
		FAFrameBeware:Show();
		FAFrameReady:Hide();
	end
	FAFrameCooling:Hide();
end

local ticker = nil;
local function StartTimer()
	if (ticker) then
		ticker:Cancel();
	end
	ticker = C_Timer.NewTicker(FLAMEBUFFICD, ICDExpired, 1);
end

local function BuffCheck(count)
	--HeatCheck();
	if FAVars.debugmode then print("buff check: count "..count.." last "..lastcount); end
	if (count ~= lastcount) then
		if (count > lastcount) then -- GAINED ONE!
			local timenow = GetTime();
			StartTimer();
			PlaySoundFile(FAVars.soundfile);
			if (lastgain and (lastgain + FLAMEBUFFICD > timenow)) then
				print("FA: second buff gained after "..(timenow-lastgain));
			end
			lastgain = timenow;
			idle = false;
			FAFrameReady:Hide();
			FAFrameBeware:Hide();
			FAFrameCooling:Show();
		end
		if (count == 30) then
			PlaySound("A_MTERRACE_SELIMEMPOWERED");
		end
		lastcount = count;
		FAFrameCounter:SetText(count);
	end
end

local function HeatCheck(heat)
	if FAVars.debugmode then print("heat check "..heat.." time "..(GetTime()-lastgain)); end
	if (not overheat and heat > FAVars.heatwarn) then
		if (FAVars.debugmode) then print("flamethrower overheating"); end
		overheat = true;
		ICDExpired(); -- change bg colour to amber
		PlaySound("A_CA_Sartharion_FireSpecial03");
	end
	overheat = heat > FAVars.heatwarn;
	if (not idle and heat < 10 and GetTime()-lastgain>10.0) then
		idle = true;
		ICDExpired(); -- change bg colour back to green
		if FAVars.debugmode then print("flamethrower idle"); end
	end
	FAFrameHeat:SetText(heat);
end

local function handler(msg)
    local command, arg = msg:match("^(%S*)%s*(.-)$");
	if (command == "debug") then
		FAVars.debugmode = not FAVars.debugmode;
		print("debug is : |cFF00FF00" .. (FAVars.debugmode and "true" or "false"));
    elseif (command == "gain") then
		BuffCheck(lastcount+1);
		return;
    elseif (command == "lose") then
		BuffCheck(lastcount-1);
		return;
	elseif (command =="stack") then
		BuffCheck(tonumber(arg));
		return;
	elseif 	(command == "heat") then
		HeatCheck(tonumber(arg));
		return;
	elseif (command == "reset") then
		BuffCheck(0);
		HeatCheck(0);
		return;
	elseif (command == "sound") then
		if (arg and strlen(arg)>0 and PlaySoundFile(arg)) then --check that its valid
			FAVars.soundfile = arg;
		end
	elseif (command == "debuff") then
		if (arg and strlen(arg)>0) then
			FAVars.buffname = arg;
		end
	elseif (command == "warn") then
		if (arg and strlen(arg)>0) then
			FAVars.heatwarn = tonumber(arg);
		end
	elseif (command == "defaults") then
		FAVars = defaults;
	else
		print("|cFFFF0000Flame Assist|r");
		print("|cFF00FF00Options: /fa sound <file> : |rChange the sound used for gaining a stack");
        print("|cFF00FF00Options: /fa debuff <name> : |rChange the name of the stacking debuff to track");
		print("|cFF00FF00Options: /fa warn <value> : |rChange the heat level to switch to warning colour");
		print("|cFF00FF00Options: /fa defaults : |rReset all options to default");
        print("|cFFFFFF00Debug Options: /fa debug gain lose stack heat reset");
        print("Left click to drag, right click to resize");
        print("");
    end
	print("current sound file is : |cFF00FF00",FAVars.soundfile);
	print("current debuff name is : |cFF00FF00",FAVars.buffname);
	print("current warn is : |cFF00FF00"..FAVars.heatwarn);
end

SLASH_FASLASH1, SLASH_FASLASH2 = '/fa', '/flameassist';
SlashCmdList["FASLASH"] = handler;

function FA_OnLoad(self)
-- init saved vars table
    if not FAVars then
        FAVars = defaults;
    end
-- make draggable
	self:RegisterForDrag("LeftButton");
	self:SetMinResize(16,16);
-- register for all events we want
    self:RegisterEvent("UNIT_AURA");
    self:RegisterEvent("UNIT_POWER");
    self:RegisterEvent("PLAYER_ENTERING_WORLD");
end

function FA_OnEvent(self, event, ...)
	if (event == "UNIT_AURA") then
		local unitname = ...;
		--if FAVars.debugmode then print("aura change "..unitname); end
		if (unitname == "player") then
			local ft = select(6,UnitBuff("player","Flamethrower"));
			local pickedup = ft ~= nil;
			if (enabled and not pickedup and not FAVars.debugmode) then
				enabled = false;
				FAFrame:Hide();
				if FAVars.debugmode then print("flamethrower dropped"); end
			end
			if (not enabled and (pickedup or FAVars.debugmode)) then
				enabled = true;
				overheat = false;
				FAFrame:Show();
				FAFrameCounter:SetText("0");
				FAFrameHeat:SetText("0");
				if FAVars.debugmode then print("flamethrower picked up"); end
			end
			if (enabled) then
				local count = select(4,UnitDebuff("player",FAVars.buffname));
				if (count == nil and lastcount>0) then -- stack has decayed or died?
					count = 0;
					if FAVars.debugmode then print("stack dropped"); end
				end
				if (count ~= nil) then
					BuffCheck(count);
				end
			end
		end
	elseif (event == "UNIT_POWER") then
		local unitname,ptype = ...;
		if (unitname == "player") then
			--if FAVars.debugmode then print("unit power "..ptype); end
			if (ptype == "ALTERNATE") then
				HeatCheck(UnitPower("player",ALTERNATE_POWER_INDEX));
			end
		end
	elseif (event == "PLAYER_ENTERING_WORLD") then
		FAFrameCounter:SetTextHeight(self:GetHeight()/2); -- doesnt work in onload :/
		FAFrameCounter:SetText("0");		
		FAFrameHeat:SetText("0");
	end
end

function FA_StartSizing()
	if (not InCombatLockdown()) then
		FAFrame:StartSizing();
	end
end
